/*
 * Copyright 2011 Free Software Foundation, Inc.
 *
 * This file was generated by gr_modtool, a tool from the GNU Radio framework
 * This file is a part of gr-howto
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

#ifndef INCLUDED_HOWTO_API_H
#define INCLUDED_HOWTO_API_H

#include <gnuradio/attributes.h>

#ifdef gnuradio_howto_EXPORTS
#define HOWTO_API __GR_ATTR_EXPORT
#else
#define HOWTO_API __GR_ATTR_IMPORT
#endif

#endif /* INCLUDED_HOWTO_API_H */
